#include "font.hpp"
#include <string>
using namespace std;

#include <boost/lexical_cast.hpp>

/* function to recover memory form our list of characters */
FontRenderer::~FontRenderer( GLvoid )
{
    glDeleteLists( base, 256 ); /* Delete All 256 Display Lists */
    glDeleteLists( smallbase, 256 ); /* Delete All 256 Display Lists */

    return;
}



FontRenderer::FontRenderer( )
{
    /* Status indicator */
    int Status = 0;

    //LOAD THE BIG FONT
    {
        /* Create storage space for the texture */
        SDL_Surface *TextureImage;

        /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
        if ( TextureImage = SDL_LoadBMP( "textures/font.bmp" ) )
        {


            /* Set the status to true */
            Status = 1;

            /* Create The Texture */
            glGenTextures( 1, &font_texture );

            /* Load in texture 1 */
            /* Typical Texture Generation Using Data From The Bitmap */
            glBindTexture( GL_TEXTURE_2D, font_texture );

            /* Generate The Texture */
            glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w,
                         TextureImage->h, 0, GL_BGR,
                         GL_UNSIGNED_BYTE, TextureImage->pixels );

            /* Nearest Filtering */
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

        }

        /* Free up any memory we may have used */
        if ( TextureImage )
            SDL_FreeSurface( TextureImage );
    }

    //LOAD THE SMALL FONT
    {
        /* Create storage space for the texture */
        SDL_Surface *TextureImage;

        /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
        if ( TextureImage = SDL_LoadBMP( "textures/smallfont.bmp" ) )
        {


            /* Set the status to true */
            Status = 1;

            /* Create The Texture */
            glGenTextures( 1, &smallfont_texture );

            /* Load in texture 1 */
            /* Typical Texture Generation Using Data From The Bitmap */
            glBindTexture( GL_TEXTURE_2D, smallfont_texture );

            /* Generate The Texture */
            glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w,
                         TextureImage->h, 0, GL_BGR,
                         GL_UNSIGNED_BYTE, TextureImage->pixels );

            /* Nearest Filtering */
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

        }

        /* Free up any memory we may have used */
        if ( TextureImage )
            SDL_FreeSurface( TextureImage );
    }

    BuildFont();

}

string format_grouping (int value)
{
  string result;

  if (value==0) return "0";

  while (true)
  {
    for (int i=0; i<3; ++i)
	  {result=char('0'+value%10)+result;value/=10;if (value==0) break;}

	if (value>0) result=","+result; else break;
  }
  return result;
}


/* function to build our font list */
void  FontRenderer::BuildFont()
{
    //1: build the big font
    {
        GLuint loop; /* Loop variable               */
        float cx;    /* Holds Our X Character Coord */
        float cy;    /* Holds Our Y Character Coord */

        /* Creating 256 Display List */
        base  = glGenLists( 256 );
        /* Select Our Font Texture */
        glBindTexture( GL_TEXTURE_2D, font_texture );

        /* Loop Through All 256 Lists */
        for ( loop = 0; loop < 256; loop++ )
        {
            /* NOTE:
             *  BMPs are stored with the top-leftmost pixel being the
             * last byte and the bottom-rightmost pixel being the first
             * byte. So an image that is displayed as
             *    1 0
             *    0 0
             * is represented data-wise like
             *    0 0
             *    0 1
             * And because SDL_LoadBMP loads the raw data without
             * translating to how it is thought of when viewed we need
             * to start at the bottom-right corner of the data and work
             * backwards to get everything properly. So the below code has
             * been modified to reflect this. Examine how this is done and
             * how the original tutorial is done to grasp the differences.
             *
             * As a side note BMPs are also stored as BGR instead of RGB
             * and that is why we load the texture using GL_BGR. It's
             * bass-ackwards I know but whattaya gonna do?
             */

            /* X Position Of Current Character */
            cx = 1 - ( float )( loop % 16 ) / 16.0f;
            /* Y Position Of Current Character */
            cy = 1 - ( float )( loop / 16 ) / 16.0f;

            /* Start Building A List */
            glNewList( base + ( 255 - loop ), GL_COMPILE );

        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
            glBegin( GL_QUADS );
            /* Texture Coord (Bottom Left) */
            glTexCoord2f( cx - 0.0625, cy );
            /* Vertex Coord (Bottom Left) */
            glVertex2i( 0, 0 );

            /* Texture Coord (Bottom Right) */
            glTexCoord2f( cx, cy );
            /* Vertex Coord (Bottom Right) */
            glVertex2i( 16, 0 );

            /* Texture Coord (Top Right) */
            glTexCoord2f( cx, cy - 0.0625f );
            /* Vertex Coord (Top Right) */
            glVertex2i( 16, 16 );

            /* Texture Coord (Top Left) */
            glTexCoord2f( cx - 0.0625f, cy - 0.0625f);
            /* Vertex Coord (Top Left) */
            glVertex2i( 0, 16 );
            glEnd( );

            /* Move To The Left Of The Character */
            glTranslated( 16, 0, 0 );
                    glDisable(GL_TEXTURE_2D);

            glEndList( );
        }
    }

     //2: build the small font
    {
        GLuint loop; /* Loop variable               */
        float cx;    /* Holds Our X Character Coord */
        float cy;    /* Holds Our Y Character Coord */

        /* Creating 256 Display List */
        smallbase  = glGenLists( 256 );
        /* Select Our Font Texture */
        glBindTexture( GL_TEXTURE_2D, smallfont_texture );

        /* Loop Through All 256 Lists */
        for ( loop = 0; loop < 256; loop++ )
        {
            /* NOTE:
             *  BMPs are stored with the top-leftmost pixel being the
             * last byte and the bottom-rightmost pixel being the first
             * byte. So an image that is displayed as
             *    1 0
             *    0 0
             * is represented data-wise like
             *    0 0
             *    0 1
             * And because SDL_LoadBMP loads the raw data without
             * translating to how it is thought of when viewed we need
             * to start at the bottom-right corner of the data and work
             * backwards to get everything properly. So the below code has
             * been modified to reflect this. Examine how this is done and
             * how the original tutorial is done to grasp the differences.
             *
             * As a side note BMPs are also stored as BGR instead of RGB
             * and that is why we load the texture using GL_BGR. It's
             * bass-ackwards I know but whattaya gonna do?
             */

            /* X Position Of Current Character */
            cx = 1 - ( float )( loop % 16 ) / 16.0f;
            /* Y Position Of Current Character */
            cy = 1 - ( float )( loop / 16 ) / 16.0f;

            /* Start Building A List */
            glNewList( smallbase + ( 255 - loop ), GL_COMPILE );

        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
            glBegin( GL_QUADS );
            /* Texture Coord (Bottom Left) */
            glTexCoord2f( cx - 0.0625f, cy );
            /* Vertex Coord (Bottom Left) */
            glVertex2i( 0, 0 );

            /* Texture Coord (Bottom Right) */
            glTexCoord2f( cx, cy );
            /* Vertex Coord (Bottom Right) */
            glVertex2i( 8, 0 );

            /* Texture Coord (Top Right) */
            glTexCoord2f( cx, cy - 0.0625f );
            /* Vertex Coord (Top Right) */
            glVertex2i( 8, 8 );

            /* Texture Coord (Top Left) */
            glTexCoord2f( cx - 0.0625f, cy - 0.0625f);
            /* Vertex Coord (Top Left) */
            glVertex2i( 0, 8 );
            glEnd( );

            /* Move To The Left Of The Character */
            glTranslated( 8, 0, 0 );
                                glDisable(GL_TEXTURE_2D);

            glEndList( );
        }
    }

}


GLvoid FontRenderer::glSmallPrint( GLint x, GLint y, const char *string, int set )
{
    if ( set > 1 )
		set = 1;

    /* Select our texture */
    glBindTexture( GL_TEXTURE_2D, smallfont_texture );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Store The Projection Matrix */
    glPushMatrix( );

    /* Reset The Projection Matrix */
    glLoadIdentity( );
    /* Set Up An Ortho Screen */
    glOrtho( 0, 640, 0, 480, -1, 1 );

    /* Select The Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Stor the Modelview Matrix */
    glPushMatrix( );
    /* Reset The Modelview Matrix */
    glLoadIdentity( );

    /* Position The Text (0,0 - Bottom Left) */
    glTranslated( x, y, 0 );

    /* Choose The Font Set (0 or 1) */
    glListBase( smallbase - 32 + ( 128 * set ) );

    /* Write The Text To The Screen */
    glCallLists( strlen( string ), GL_BYTE, string );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Restore The Old Projection Matrix */
    glPopMatrix( );

    /* Select the Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Restore the Old Projection Matrix */
    glPopMatrix( );
}

GLvoid FontRenderer::glSmallPrintMultiLine( GLint x, GLint y, int width, const char* str, int set )
{

    if (width==0)
        return;

    string s=str;
    int cury=y;
    unsigned int startp=0;
    while (startp<s.size())
    {
        int endp=startp+1;
        int lastspace=-1;

        while (endp-startp<=width)
        {

            if (endp==s.size())
            {
                lastspace=endp;
                break;
            }
            if (s[endp]=='\n')
            {
                lastspace=endp;
                break;
            }
            else if (s[endp]==' ')
            {
                lastspace=endp;
            }
            endp++;
        }

        if (lastspace>=0)//wrap a word around
        {
            //if two consecutive newlines
            if (s.substr(startp,lastspace-startp)=="\n")
            {
                startp=lastspace+1;
            }
            else
            {
                glSmallPrint(x,cury,s.substr(startp,lastspace-startp).c_str(),0);
                startp=lastspace+1;
            }
        }
        else //word bigger than line, just thunk
        {
            glSmallPrint(x,cury,s.substr(startp,endp-startp).c_str(),0);
            startp+=width;
        }

        cury-=10;

    }
}



GLvoid FontRenderer::glSmallPrintNum( GLint x, GLint y, int n)
{
    string s = format_grouping(n);

    glSmallPrint(x-8*s.size(),y,s.c_str(),0);

}

GLvoid FontRenderer::glSmallPrintAlignRight(  GLint x, GLint y, const char *string, int set )
{
    glSmallPrint(x-8*strlen(string),y,string,set);
}

GLvoid FontRenderer::glPrint( GLint x, GLint y, const char *string, int set )
{
    if ( set > 1 )
		set = 1;

    /* Select our texture */
    glBindTexture( GL_TEXTURE_2D, font_texture );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Store The Projection Matrix */
    glPushMatrix( );

    /* Reset The Projection Matrix */
    glLoadIdentity( );
    /* Set Up An Ortho Screen */
    glOrtho( 0, 640, 0, 480, -1, 1 );

    /* Select The Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Stor the Modelview Matrix */
    glPushMatrix( );
    /* Reset The Modelview Matrix */
    glLoadIdentity( );

    /* Position The Text (0,0 - Bottom Left) */
    glTranslated( x, y, 0 );

    /* Choose The Font Set (0 or 1) */
    glListBase( base - 32 + ( 128 * set ) );

    /* Write The Text To The Screen */
    glCallLists( strlen( string ), GL_BYTE, string );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Restore The Old Projection Matrix */
    glPopMatrix( );

    /* Select the Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Restore the Old Projection Matrix */
    glPopMatrix( );

}
