#include "graphics.hpp"

void GraphicsEngine::flip()
{
		SDL_GL_SwapBuffers();
}

GLuint GraphicsEngine::loadmodel(const string& mesh,const string& texture)
{
  Model* m = new Model;
  m->load(mesh.c_str(),texture.c_str());
  models.push_back(m);
  return m->dlist;
}

void GraphicsEngine::drawpane(int x1, int y1, int x2, int y2)
{
         glDisable(GL_BLEND);
        glColor4f(0.0,0.0,0.0,1.0);
        glBegin(GL_QUADS);
        glVertex2f(x1,y1);
        glVertex2f(x1,y2);
        glVertex2f(x2,y2);
        glVertex2f(x2,y1);
        glEnd();
        glEnable(GL_BLEND);

        glColor3f(0.5f,0.5f,1.0f);
        glBegin(GL_LINE_LOOP);
        glVertex2f(x1,y1);
        glVertex2f(x1,y2);
        glVertex2f(x2,y2);
        glVertex2f(x2,y1);
        glEnd();
}

int GraphicsEngine::resizeWindow( int width, int height )
{
    /* Height / width ration */
    GLfloat ratio;

    /* Protect against a divide by zero */
    if ( height == 0 )
		height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLint )width, ( GLint )height );

    /*
     * change to the projection matrix and set
     * our viewing volume.
     */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Set our perspective */
    glOrtho( 0, 640, 480, 0, -1, 1 );

    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode( GL_MODELVIEW );

    /* Reset The View */
    glLoadIdentity( );

    return( 1 );
}

GraphicsEngine::GraphicsEngine()
{

  int videoFlags;
  const SDL_VideoInfo *videoInfo;

  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    std::cout<<"Video initialization failed: " << SDL_GetError( ) << std::endl;
	}

  videoInfo = SDL_GetVideoInfo( );

  if ( !videoInfo )
	{
	    std::cout<<"Video query failed: " << SDL_GetError( ) << std::endl;
	}

    /* the flags to pass to SDL_SetVideoMode */
  videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
  videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
  videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
  videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

  /* This checks to see if surfaces can be stored in memory */
  if ( videoInfo->hw_available )
    videoFlags |= SDL_HWSURFACE;
  else
    videoFlags |= SDL_SWSURFACE;

  /* This checks if hardware blits can be done */
  if ( videoInfo->blit_hw )
    videoFlags |= SDL_HWACCEL;

  /* Sets up OpenGL double buffering */
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 2 );

  /* get a SDL surface */
  screen = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
                videoFlags | SDL_FULLSCREEN);

	SDL_ShowCursor(SDL_DISABLE);

  /* Verify there is a surface */
  if ( !screen )
	{
	    std::cout<< "Video mode set failed: " << SDL_GetError( ) << std::endl;
	}

	initGL();

  resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

  fontrenderer = new FontRenderer();
  iconrenderer = new IconRenderer();

}


const char* shadercode[]=
{"void main()",
"{",
"	// Setting Each Pixel To Red",
"	gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);",
"}"};

void GraphicsEngine::initGL()
{


//void glShaderSourceARB(GLhangleARB shader, GLuint number_strings, const GLcharARB** strings, Glint * length);

    /* Enable Texture Mapping ( NEW ) */
    glEnable( GL_TEXTURE_2D );

    glEnable (GL_BLEND);

 glBlendFunc(GL_ONE, GL_ONE);
//    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


    /* Enable smooth shading */
    glShadeModel( GL_SMOOTH );
    glEnable (GL_LINE_SMOOTH);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);

    glLineWidth(1.5);

    /* Set the background black */
    glClearColor( 0.0f, 0.0f, 0.0f, 0.5f );

	glMatrixMode(GL_TEXTURE);
	glRotatef(180.0f,0.0f,0.0f,1.0f);
	glScalef(-1.0f,1.0f,1.0f);

	glMatrixMode(GL_MODELVIEW);

	// Load The Mesh Data
  g_pMesh = new CMesh();							// Instantiate Our Mesh

  g_pMesh->LoadHeightmap();
	// Check For VBOs Supported
  g_fVBOSupported = IsExtensionSupported( "GL_ARB_vertex_buffer_object" );
	if( g_fVBOSupported )
	{
		#ifndef __APPLE__
		// Get Pointers To The GL Functions
		glGenBuffersARB = (PFNGLGENBUFFERSARBPROC) wglGetProcAddress("glGenBuffersARB");
		glBindBufferARB = (PFNGLBINDBUFFERARBPROC) wglGetProcAddress("glBindBufferARB");
		glBufferDataARB = (PFNGLBUFFERDATAARBPROC) wglGetProcAddress("glBufferDataARB");
		glDeleteBuffersARB = (PFNGLDELETEBUFFERSARBPROC) wglGetProcAddress("glDeleteBuffersARB");
		#endif
		// Load Vertex Data Into The Graphics Card Memory
		g_pMesh->BuildVBOs();						// Build The VBOs
	}


}
