#include "icons.hpp"


/* function to recover memory form our list of characters */
IconRenderer::~IconRenderer( GLvoid )
{
    glDeleteLists( base, 4 ); /* Delete All 256 Display Lists */
    return;
}

IconRenderer::IconRenderer( )
{
    /* Status indicator */
    int Status = 0;

    //LOAD THE ICON TEXTURE
    {
        /* Create storage space for the texture */
        SDL_Surface *TextureImage;

        /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
        if ( TextureImage = SDL_LoadBMP( "textures/icons.bmp" ) )
        {


            /* Set the status to true */
            Status = 1;

            /* Create The Texture */
            glGenTextures( 1, &icon_texture );

            /* Load in texture 1 */
            /* Typical Texture Generation Using Data From The Bitmap */
            glBindTexture( GL_TEXTURE_2D, icon_texture );

            /* Generate The Texture */
            glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w,
                          TextureImage->h, 0, GL_BGR,
                          GL_UNSIGNED_BYTE, TextureImage->pixels );

            /* Nearest Filtering */
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

        }

        /* Free up any memory we may have used */
        if ( TextureImage )
            SDL_FreeSurface( TextureImage );
    }

    //LOAD THE AVATAR TEXTURE
    {
        /* Create storage space for the texture */
        SDL_Surface *TextureImage;

        /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
        if ( TextureImage = SDL_LoadBMP( "textures/avatars.bmp" ) )
        {


            /* Set the status to true */
            Status = 1;

            /* Create The Texture */
            glGenTextures( 1, &avatar_texture );

            /* Load in texture 1 */
            /* Typical Texture Generation Using Data From The Bitmap */
            glBindTexture( GL_TEXTURE_2D, avatar_texture );

            /* Generate The Texture */
            glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w,
                          TextureImage->h, 0, GL_BGR,
                          GL_UNSIGNED_BYTE, TextureImage->pixels );

            /* Nearest Filtering */
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
            glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

        }

        /* Free up any memory we may have used */
        if ( TextureImage )
            SDL_FreeSurface( TextureImage );
    }

    BuildIcon();
    BuildAvatars();
}

static void fastline(GLfloat x1,GLfloat y1,GLfloat x2,GLfloat y2)
{
    glBegin(GL_LINES);
    glVertex2f(x2,y2);
    glVertex2f(x1,y1);
    glEnd();
}


static void outline()
{
    fastline(0,0,0,24);
    fastline(26,0,26,24);
    fastline(0,24,26,24);
}

/* function to build our font list */
void  IconRenderer::BuildIcon()
{
    base  = glGenLists( 5 );
    /* Select Our Font Texture */
    glBindTexture( GL_TEXTURE_2D, icon_texture );

    {
        /* Start Building A List */
        glNewList( base, GL_COMPILE );

        outline();

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0, tilesize );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0+4, 0+2 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( tilesize, tilesize );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 18+4, 0+2 );

        /* Texture Coord (Top Right) */
        glTexCoord2f( tilesize, 0 );
        /* Vertex Coord (Top Right) */
        glVertex2i( 18+4, 18+2 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0, 0);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0+4, 18+2 );
        glEnd( );

        glDisable(GL_TEXTURE_2D);


        glEndList( );
    }
    {
        /* Start Building A List */
        glNewList( base+1, GL_COMPILE );

        outline();

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0+tilesize, tilesize );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0+4, 0+2 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( tilesize+tilesize, tilesize );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 18+4, 0+2);

        /* Texture Coord (Top Right) */
        glTexCoord2f( tilesize+tilesize, 0 );
        /* Vertex Coord (Top Right) */
        glVertex2i( 18+4, 18+2 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0+tilesize, 0);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0+4, 18+2 );
        glEnd( );
        glDisable(GL_TEXTURE_2D);

        glEndList( );
    }


    {
        /* Start Building A List */
        glNewList( base+2, GL_COMPILE );

        outline();

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0, tilesize+tilesize );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0+4, 0+2 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( tilesize, tilesize+tilesize );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 18+4, 0+2 );

        /* Texture Coord (Top Right) */
        glTexCoord2f( tilesize, 0 +tilesize);
        /* Vertex Coord (Top Ri ght) */
        glVertex2i( 18+4, 18+2 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0, 0+tilesize);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0+4, 18+2 );
        glEnd( );
        glDisable(GL_TEXTURE_2D);

        glEndList( );
    }
    {
        /* Start Building A List */
        glNewList( base+3, GL_COMPILE );

        outline();

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0+tilesize, tilesize+tilesize );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0+4, 0 +2);

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( tilesize+tilesize, tilesize +tilesize);
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 18+4, 0+2 );

        /* Texture Coord (Top Right) */
        glTexCoord2f( tilesize+tilesize, 0 +tilesize);
        /* Vertex Coord (Top Right) */
        glVertex2i( 18+4, 18+2 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0+tilesize, 0+tilesize);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0+4, 18+2 );
        glEnd( );
        glDisable(GL_TEXTURE_2D);

        glEndList( );
    }


    {
        /* Start Building A List */
        glNewList( base+4, GL_COMPILE );

        outline();

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0+tilesize+tilesize, tilesize );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0+4, 0+2 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( tilesize+tilesize+tilesize, tilesize );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 18+4, 0+2);

        /* Texture Coord (Top Right) */
        glTexCoord2f( tilesize+tilesize+tilesize, 0 );
        /* Vertex Coord (Top Right) */
        glVertex2i( 18+4, 18+2 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0+tilesize+tilesize, 0);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0+4, 18+2 );
        glEnd( );
        glDisable(GL_TEXTURE_2D);

        glEndList( );
    }
}


/* function to build our avatars */
void  IconRenderer::BuildAvatars()
{
    avatarbase  = glGenLists( 2 );
    /* Select Our Font Texture */
    glBindTexture( GL_TEXTURE_2D, avatar_texture );

    {
        /* Start Building A List */
        glNewList( avatarbase, GL_COMPILE );


        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0, 1 );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0, 0 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( 0.5, 1 );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 85, 0 );

        /* Texture Coord (Top Right) */
        glTexCoord2f( 0.5, 0 );
        /* Vertex Coord (Top Right) */
        glVertex2i( 85, 119 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0, 0);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0, 119 );
        glEnd( );

        glDisable(GL_TEXTURE_2D);


        glEndList( );
    }
    {
             /* Start Building A List */
        glNewList( avatarbase+1, GL_COMPILE );

        float tilesize=(1.0f/64.0f)*18.0f;
        glEnable(GL_TEXTURE_2D);
        /* Use A Quad For Each Character */
        glBegin( GL_QUADS );
        /* Texture Coord (Bottom Left) */
        glTexCoord2f(0.5, 1 );
        /* Vertex Coord (Bottom Left) */
        glVertex2i( 0, 0 );

        /* Texture Coord (Bottom Right) */
        glTexCoord2f( 1, 1 );
        /* Vertex Coord (Bottom Right) */
        glVertex2i( 85, 0 );

        /* Texture Coord (Top Right) */
        glTexCoord2f( 1, 0 );
        /* Vertex Coord (Top Right) */
        glVertex2i( 85, 119 );

        /* Texture Coord (Top Left) */
        glTexCoord2f( 0.5, 0);
        /* Vertex Coord (Top Left) */
        glVertex2i( 0, 119 );
        glEnd( );

        glDisable(GL_TEXTURE_2D);


        glEndList( );
    }
}


void IconRenderer::glAvatar(GLint x, GLint y, int av)
{
 /* Select our texture */
    glBindTexture( GL_TEXTURE_2D, avatar_texture );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Store The Projection Matrix */
    glPushMatrix( );

    /* Reset The Projection Matrix */
    glLoadIdentity( );
    /* Set Up An Ortho Screen */
    glOrtho( 0, 640, 0, 480, -1, 1 );

    /* Select The Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Stor the Modelview Matrix */
    glPushMatrix( );
    /* Reset The Modelview Matrix */
    glLoadIdentity( );

    /* Position The Text (0,0 - Bottom Left) */
    glTranslated( x, y, 0 );

    glCallList(avatarbase+2-av);

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Restore The Old Projection Matrix */
    glPopMatrix( );

    /* Select the Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Restore the Old Projection Matrix */
    glPopMatrix( );
}

void IconRenderer::glIcon( GLint x, GLint y, int icon )
{
    /* Select our texture */
    glBindTexture( GL_TEXTURE_2D, icon_texture );

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Store The Projection Matrix */
    glPushMatrix( );

    /* Reset The Projection Matrix */
    glLoadIdentity( );
    /* Set Up An Ortho Screen */
    glOrtho( 0, 640, 0, 480, -1, 1 );

    /* Select The Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Stor the Modelview Matrix */
    glPushMatrix( );
    /* Reset The Modelview Matrix */
    glLoadIdentity( );

    /* Position The Text (0,0 - Bottom Left) */
    glTranslated( x, y, 0 );

    glCallList(base+icon);

    /* Select The Projection Matrix */
    glMatrixMode( GL_PROJECTION );
    /* Restore The Old Projection Matrix */
    glPopMatrix( );

    /* Select the Modelview Matrix */
    glMatrixMode( GL_MODELVIEW );
    /* Restore the Old Projection Matrix */
    glPopMatrix( );

}
