#include <iostream>

#include "model.hpp"

#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_confix.hpp>
#include <boost/spirit/include/classic_chset.hpp>
#include <boost/spirit/include/classic_actor.hpp>
#include <boost/spirit/include/classic_file_iterator.hpp>
#include <boost/bind.hpp>


using namespace boost::spirit;
using namespace BOOST_SPIRIT_CLASSIC_NS;



typedef file_iterator<char> iterator_t;
typedef scanner<iterator_t> scanner_t;
typedef rule<scanner_t> rule_t;


vector<GLfloat> tempvertices_flat;
vector<vector<int> > tempfaces;//contains list of points (vertex1,textcoord1,vertex2,textcoord2,vertex2,textcoord3)
vector<GLfloat> temptexturecoords_flat;



void addFirstFD(const double d)
{
	vector<int> blank;
	for (int i=0;i<6;i++)
		blank.push_back(0);

	tempfaces.push_back(blank);

	tempfaces[tempfaces.size()-1][0]=(int)d;

	//texture-coord pointers stored here:
	tempfaces[tempfaces.size()-1][1]=(temptexturecoords_flat.size()/2-1);
	tempfaces[tempfaces.size()-1][3]=(temptexturecoords_flat.size()/2-2);
	tempfaces[tempfaces.size()-1][5]=(temptexturecoords_flat.size()/2-3);

	//faces.push_back(blank);*/
}

void addSecondFD(const double d)
{
	tempfaces[tempfaces.size()-1][2]=(int)d;

	//faces.push_back(blank);
}

void addThirdFD(const double d)
{
	tempfaces[tempfaces.size()-1][4]=(int)d;

	//faces.push_back(blank);
}

struct ModelParse : public grammar<ModelParse>
{
	template <typename ScannerT>
	struct definition
	{
		definition(ModelParse const& self)
		{
			ObjFile_p= +(vertices_p | vtex_p | faces_p | point_p  );

			vertices_p = 'v'
			>> real_p[push_back_a(tempvertices_flat)]
			>> real_p[push_back_a(tempvertices_flat)]
			>> real_p[push_back_a(tempvertices_flat)];

			vtex_p = "vt"
			>> real_p[push_back_a(temptexturecoords_flat)]
			>> real_p[push_back_a(temptexturecoords_flat)];

			faces_p = 'f'
			>> real_p[&addFirstFD]>>'/'>>real_p
			>> real_p[&addSecondFD]>>'/'>>real_p
			>> real_p[&addThirdFD]>>'/'>>real_p;

			point_p = 'p'
			>> real_p>>'/'>>real_p;
		}

		rule<ScannerT> 	ObjFile_p, vertices_p, vtex_p, faces_p,point_p;

		rule<ScannerT> const&
		start() const { return ObjFile_p; }
	};
};

void Model::render()
{
	dlist=glGenLists(1);

	glNewList(dlist,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_TRIANGLES);
	for (unsigned int i=0;i<faces.size();i++)
	{
		/*
		std::cout<<"\nTRIANGLE "<<i<<"\n";
		std::cout<<texturecoords_flat[2*(faces[i][1])]<<","
			<<texturecoords_flat[2*(faces[i][1])+1]<<"\n";
		std::cout<<texturecoords_flat[2*(faces[i][3])]<<","
		<<texturecoords_flat[2*(faces[i][3])+1]<<"\n";
		std::cout<<texturecoords_flat[2*(faces[i][5])]<<","
		<<texturecoords_flat[2*(faces[i][5])+1]<<"\n";
		*/
		glTexCoord2fv(&texturecoords_flat[2*(faces[i][1])]);

		glVertex3fv(&vertices_flat[3*(faces[i][0]-1)]);

		glTexCoord2fv(&texturecoords_flat[2*(faces[i][3])]);

		glVertex3fv(&vertices_flat[3*(faces[i][2]-1)]);

		glTexCoord2fv(&texturecoords_flat[2*(faces[i][5])]);

		glVertex3fv(&vertices_flat[3*(faces[i][4]-1)]);
	}
	glEnd();
	glEndList();
}

bool Model::load(const char* filename, const char* texture)
{
	tempfaces.clear();
	tempvertices_flat.clear();
	temptexturecoords_flat.clear();

	// Create a file iterator for this file
    iterator_t first(filename);

    if (!first)
    {
        std::cout << "Unable to open file!\n";
        return -1;
    }

    // Create an EOF iterator
    iterator_t last = first.make_end();

	ModelParse g;
	bool res = parse(first,last,

				 //  Begin grammar
				 g
				 ,
				 //  End grammar

				 (
					space_p|comment_p("#") |comment_p("usemtl")|comment_p("o")
					 )
	).full;

	vertices_flat=tempvertices_flat;
	faces=tempfaces;
	texturecoords_flat=temptexturecoords_flat;

	loadTexture(texture);
	render();

	return res;
	//now copy the temp
}

void Model::loadTexture(const char* texture_path)
{
	/* Create storage space for the texture */
    SDL_Surface *TextureImage;

    /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
    if ( TextureImage = SDL_LoadBMP( texture_path ) )
	{
	    /* Create The Texture */
	    glGenTextures( 1, &texture );

	    /* Typical Texture Generation Using Data From The Bitmap */
	    glBindTexture( GL_TEXTURE_2D, texture );

	    /* Generate The Texture */
	    glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w,
					 TextureImage->h, 0, GL_BGR,
					 GL_UNSIGNED_BYTE, TextureImage->pixels );

	    /* Linear Filtering */
	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

		SDL_FreeSurface( TextureImage );
	}


	//back to normal
	glMatrixMode(GL_MODELVIEW);

}
