#ifndef WIDGET_HPP
#define WIDGET_HPP

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include <math.h>

#include "Engine/icons.hpp"
#include "Engine/font.hpp"

/*
What do I need:

1: button tied to variable (for three options)
2: click button
3: slider area
5: dialogue box

*/

class Screen_Element {
public:
	int x;
	int y;
	int w;
	int h;
	char *tooltip;
	bool focused;
	bool enabled;
    bool on;
	virtual void enable(){enabled=true;};
	virtual void draw()=0;
	virtual void mouseinput(int mx, int my, bool mb1){};//returns true if wants to capture the mouse
	virtual void uncapture(){};
	bool isat(int px, int py);
	Screen_Element(): focused(false) {};
	Screen_Element(int ex, int ey, int ew, int eh, char *etooltip,bool eenabled): x(ex), y(ey), w(ew), h(eh), tooltip(etooltip),focused(false),enabled(eenabled){};
	virtual ~Screen_Element() {};
};


class Level_Select_Bar: public Screen_Element {
public:
	int boxcount;
	int& curselected;
	const int& curmaxlevel;
	void (*onclick)(int& param, bool& b);
	void draw();
	void mouseinput(int mx, int my, bool mb1);
	Level_Select_Bar(int ex, int ey, int ew, int eh, char* etooltip, int eboxcount, const int& ecurmax, int& ecurselected, bool eenabled):
	Screen_Element(ex, ey, ew, eh, etooltip,eenabled),
	boxcount(eboxcount),
	curselected(ecurselected),
	curmaxlevel(ecurmax)
	{
	};
};


enum Button_State {bs_normal, bs_down};

//for this game, don't need to allow turn off after turning on
class Set_Button : public Screen_Element {
public:
	Button_State state;
	int image;
	IconRenderer* iconrenderer;
	int& param;
	void (*onclick)(int& param, bool& b);
	virtual void draw();
	void uncapture();
	void mouseinput(int mx, int my, bool mb1);
	Set_Button(int ex, int ey, int ew, int eh, IconRenderer* eiconrenderer, int eimage, char* etooltip, bool eenabled,void (*eonclick)(int&, bool&),int& eparam,bool eon):
	Screen_Element(ex, ey, ew, eh, etooltip,eenabled),
    state(bs_normal),
	image(eimage),
	iconrenderer(eiconrenderer),
	param(eparam),
	onclick(eonclick)
	{
        on=eon;
	};
};

class Text_Button : public Set_Button {
public:
	FontRenderer* fontrenderer;
	char* text;
	void (*onclick)(bool& b);
	void draw();
	Text_Button(int ex, int ey, FontRenderer* efontrenderer, char* etext, char* etooltip, bool eenabled,void (*eonclick)(int&, bool&),int& eparam,bool eon):
	Set_Button(ex,ey,strlen(etext)*16+6,22,NULL,0,etooltip,eenabled,eonclick,eparam,eon),
	fontrenderer(efontrenderer),
    text(etext)
	{
	};
};

#endif /* GLOBALS_HPP */

